package xyz.lychee.lagfixer.modules;

import com.google.common.collect.ImmutableList;
import java.util.Collection;
import java.util.EnumSet;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import org.bukkit.entity.Animals;
import org.bukkit.entity.Entity;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Flying;
import org.bukkit.entity.Monster;
import org.bukkit.entity.Tameable;
import org.bukkit.entity.Villager;
import org.bukkit.event.EventHandler;
import org.bukkit.event.HandlerList;
import org.bukkit.event.Listener;
import org.bukkit.event.entity.CreatureSpawnEvent;
import org.bukkit.event.vehicle.VehicleCreateEvent;
import xyz.lychee.lagfixer.LagFixer;
import xyz.lychee.lagfixer.managers.ModuleManager;
import xyz.lychee.lagfixer.managers.SupportManager;
import xyz.lychee.lagfixer.objects.AbstractModule;
import xyz.lychee.lagfixer.objects.AbstractModule.Impact;
import xyz.lychee.lagfixer.utils.ReflectionUtils;

public class CustomAiModule extends AbstractModule implements Listener {
   private NMS customAi;
   private boolean animals;
   private boolean monsters;
   private boolean villagers;
   private boolean tameable;
   private boolean birds;
   private boolean others;
   private boolean async;
   private boolean forceLoad;
   private boolean clickEvent;
   private EnumSet<EntityType> blacklist;

   public CustomAiModule(LagFixer plugin, ModuleManager manager) {
      super(plugin, manager, Impact.VERY_HIGH, "CustomAi", ImmutableList.of("Replaces creature movement to optimize and reduce behavior.", "Addresses inefficiencies caused by default animal behavior like unnecessary random movements or constant looking around.", "CustomAi intervenes by disabling unnecessary PathFinders or replacing them with more efficient ones.", "Crucial in scenarios with numerous animals as even minor movements can strain server resources."), "eyJ0ZXh0dXJlcyI6eyJTS0lOIjp7InVybCI6Imh0dHA6Ly90ZXh0dXJlcy5taW5lY3JhZnQubmV0L3RleHR1cmUvOGM3NTA1ZjIyNGQ1MTY0YTExN2Q4YzY5ZjAxNWY5OWVmZjQzNDQ3MWM4YTJkZjkwNzA5NmM0MjQyYzM1MjRlOCJ9fX0=");
   }

   @EventHandler(
      ignoreCancelled = true
   )
   public void onSpawn(CreatureSpawnEvent e) {
      if (this.isEnabled(e.getEntity()) && this.canContinue(e.getLocation().getWorld())) {
         SupportManager.EXECUTOR.execute(() -> {
            this.customAi.setCustomAi(e.getEntity(), false);
         });
      }
   }

   @EventHandler(
      ignoreCancelled = true
   )
   public void onSpawn(VehicleCreateEvent e) {
      if (this.canContinue(e.getVehicle().getWorld())) {
         SupportManager.EXECUTOR.execute(() -> {
            this.customAi.setCustomAi(e.getVehicle(), false);
         });
      }
   }

   public boolean isEnabled(Entity ent) {
      if (this.blacklist.contains(ent.getType())) {
         return false;
      } else if (ent instanceof Tameable) {
         return this.tameable;
      } else if (ent instanceof Animals) {
         return this.animals;
      } else if (ent instanceof Flying) {
         return this.birds;
      } else if (ent instanceof Monster) {
         return this.monsters;
      } else {
         return ent instanceof Villager ? this.villagers : this.others;
      }
   }

   public void load() {
      this.getPlugin().getServer().getPluginManager().registerEvents(this, this.getPlugin());
      if (this.forceLoad) {
         SupportManager.EXECUTOR.execute(() -> {
            this.getAllowedWorlds().flatMap((w) -> {
               return SupportManager.getInstance().getCreatureSet(w).stream().filter(this::isEnabled);
            }).forEach((ent) -> {
               this.customAi.setCustomAi(ent, true);
            });
         });
      }

   }

   public boolean loadConfig() {
      this.customAi = (NMS)ReflectionUtils.createInstance("CustomAi", this);
      this.animals = this.getSection().getBoolean("entities.animals");
      this.monsters = this.getSection().getBoolean("entities.monsters");
      this.villagers = this.getSection().getBoolean("entities.villagers");
      this.tameable = this.getSection().getBoolean("entities.tameable");
      this.birds = this.getSection().getBoolean("entities.birds");
      this.others = this.getSection().getBoolean("entities.others");
      this.async = this.getSection().getBoolean("async");
      this.forceLoad = this.getSection().getBoolean("force_load");
      this.clickEvent = this.getSection().getBoolean("click_event");
      Stream var10001 = this.getSection().getStringList("blacklist").stream().map(String::toUpperCase);
      Map var10002 = SupportManager.ENTITIES;
      Objects.requireNonNull(var10002);
      var10001 = var10001.filter(var10002::containsKey);
      var10002 = SupportManager.ENTITIES;
      Objects.requireNonNull(var10002);
      this.blacklist = EnumSet.copyOf((Collection)var10001.map(var10002::get).collect(Collectors.toList()));
      return this.customAi != null;
   }

   public void disable() {
      HandlerList.unregisterAll(this);
   }

   public NMS getCustomAi() {
      return this.customAi;
   }

   public boolean isAnimals() {
      return this.animals;
   }

   public boolean isMonsters() {
      return this.monsters;
   }

   public boolean isVillagers() {
      return this.villagers;
   }

   public boolean isTameable() {
      return this.tameable;
   }

   public boolean isBirds() {
      return this.birds;
   }

   public boolean isOthers() {
      return this.others;
   }

   public boolean isAsync() {
      return this.async;
   }

   public boolean isForceLoad() {
      return this.forceLoad;
   }

   public boolean isClickEvent() {
      return this.clickEvent;
   }

   public EnumSet<EntityType> getBlacklist() {
      return this.blacklist;
   }

   public abstract static class NMS {
      private final CustomAiModule module;

      public NMS(CustomAiModule module) {
         this.module = module;
      }

      public abstract void setCustomAi(Entity var1, boolean var2);

      public CustomAiModule getModule() {
         return this.module;
      }
   }
}
